#!/bin/bash
# script name: get_hpss_tapeinfo.sh
export toolverstring="2.0 2020/12/28"		# Version and Date of the Tool  
						# ツールバージョンと日付

# date: 2019/08/26 - 2019/10/10	初期作成
#	2019/10/10		ボリューム差分リストを降順に変更
#	2020/01/09 - 2020/01/14 V.1.1 HPSS 7.5.3.0.0 環境対応。
#				テスト用ファイルの読み込み指定方法、取り出し方法変更。
#				HPSS_TapeCatalog_Template Excelファイルをscripsディレクトリーから出力ディレクトリーに複写する機能を追加。
#				dumppv_pvr出力の"Physical Volumes Pending Move"リストの処理追加。
#				ツールのバージョンと日付をコンソールメッセージとログに追加。
#				ログファイル名を変更。
#	2020/09/23 -2020/09/30	English text added to the comments.
#				Log Messages are changed to English.
#	2020/12/02		Correction of typo's in the messages
#
# function:
# 機能:
#		Get Tape related information from HPSS
#	    	HPSSのテープ関連の情報を纏めて取得する
#
# input: 	none
# 入力		無し
#
# output directory: 
# 出力ディレクトリー:
#	current working directory / hpssTapeInfo_yyyymmddhhmm
#		"hpssTapeInfo_" is a constant string and it can be changed with an environmental value.
#		yyyymmddhhmm is taken from the date and time when the tool is invoked.
#	実行時の現行ディレクトリー  / hpssTapeInfo_yyyymmddhhmm
#		"hpssTapeInfo_"は 固定文字列。環境変数で変更可能。
#		yyyymmddhhmm は、実行時日時により自動作成。
#
# output file:  Various
# 出力		いろいろ
#
# Parameter: 	none		executes HPSS commands and get information
# 		dierectory 	for debug-only
#
# パラメーター  指定無しの時 	hpss情報取得コマンドを実行する
#		ディレクトリー  テスト用。hpss情報取得コマンドを実行しない。
#				代わりに、既存のhpss情報取得コマンド結果ファイルをコピーして使用する。
#				コピー元になるファイルがあるディレクトリーを指定する。
# 				実行時のカレントディレクトリーの配下にあるディレクトリーにあると想定している。
#				既存ファイルのパスとファイル名はスクリプト内にハードコードされている。
#			注意 	テスト用なので、ディレクトリーが存在するか、そこにファイルがあるか、は検査していない。
#				ディレクトリーがなかったりファイルがないときの結果は保証しない。
#
# Environmental Variables
# 環境変数 
export datadir_pre="hpssTapeInfo_"		# prefix for Output Directory 
						# 出力dir名のプレフィックス
export logfile="Logfile_get_hpss_tapeinfo.txt"	# Name of the log file
						# ログファイル名
export outfilelist="outfilelist.csv"		# Name of the CSV file to contain names of the output files 
						# 出力ファイル名のリストCSVファイルの名前
						# The name should not be changed to work well with Excel macro.
						# Excel マクロと連携するため、この名前は変更しないこと。
# 
# find the path of this script and set it to AP, which is to be used as the path of awk scripts
# スクリプトのパスを見つけて、APにそれをセットする=>awkパスに使用する
cmdpathname=$0					# get the path and file name of the command invoked
						# 実行コマンドのパス・ファイル名を取得して
cmdname=$(basename $cmdpathname)		# get the name of the command (without path)
						# 実行コマンドのファイル名(パスなし)を取り出す
AP=$(dirname $cmdpathname)			# set path of the command to AP as the path for awk scripts
						# 実行コマンドのパスを取り出して、awkスクリプトパスにする
#	echo "cmdpathname	"$cmdpathname	# debug
#	echo "cmdname		"$cmdname	# debug
#	echo "AP     		"$AP		# debug
#
# get the current date and time
# 現在の日付と時刻を取得
dt_start_separator=$(date +"%Y/%m/%d")		# date with slashes    				スラッシュ入り日付
hm_start_separator=$(date +"%H:%M")		# hour and minute with colon			コロン入り時分
hms_start_separator=$(date +"%H:%M:%S")		# hour, minute and second with colons		コロン入り時分秒
dt_start=$(date +"%Y%m%d")			# date without slashes    			スラッシュ無し日付
hm_start=$(date +"%H%M")			# hour and minute without colon			コロン無し時分
hms_start=$(date +"%H%M%S")			# hour, minute and second without colons 	コロン無し時分秒
	echo  "Started at " $dt_start_separator" "$hms_start_separator	# debug		# log Start Date and Time 開始日時
#
# get current working directory(PWD), make the output directory under PWD and move to it
# カレントディレクリーを取得して、その下に情報収集dirを作成して移動
invokeddir=$(pwd)
echo $cmdname "(version "$toolverstring") Started."		# 20200111 log Command name with its version and date
								# 20200111 コマンド名とバージョン・日付を出力
datadir=$invokeddir"/"$datadir_pre$dt_start$hm_start	
	echo "Output will be stored in $datadir"		# log Outputdir
								# データ保存先を出力
mkdir $datadir
cd $datadir
#
# newly make a log file and record initial status to it
# ログファイルを新規作成して、初期状態を記録
echo $cmdname" (version "$toolverstring") started."           > $logfile
echo "Started date and time      : "$dt_start_separator" "$hms_start_separator >> $logfile
echo "awk PATH is located as     : "$AP                       >> $logfile
echo "Current dir when invoked is: "$invokeddir               >> $logfile
echo "Output will be put in      : "$datadir                  >> $logfile
#
# get a parameter from command-line and evaluate it
# パラメータ取得&評価
if [ "$1" = "" ]
then
	uc=0			# No parameter => executes HPSS commands
				# パラメータ指定なし=>hpss情報取得コマンド実行
	echo "No parameter specified. HPSS commands will be executed."         >> $logfile
else
	uc=1			# Parameter is found for debug.
				# .... it is treated ad the path of the files of the dummy HPSS output files.
	echo "Parameter is detected. It is used as the input directory which holds HPSS output files to be copied."   >> $logfile
	inputdir=$invokeddir"/"$1						# 20200109
	echo "Existing files are assuumed to be in " $inputdir "," >> $logfile	# 20200109
fi
#
# newly make a CSV file to contain names of the output files, and record date and time to it
# 取得情報リストを新規作成し、日付時刻を記録する
echo "\""$cmdname" output\","$dt_start_separator","$hm_start_separator  > $outfilelist	    
#
# ######################################################
# 1) get lshpss -pvr and extract PVR names	PVR_LIST
echo "*** "                                                          >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt lshpss -pvr process started" >> $logfile
raw_out=raw_lshpss-pvr.txt		# determine the file name for raw-output of HPSS command
					# 出力ファイル名を決定 コマンド出力未加工(生)
proc_out=proc_lshpss-pvr.txt		# determine the file name for processed-output of HPSS command
					# 出力ファイル名を決定 加工後

if [ $uc -eq 0 ] ;			# check command-line parameter 
then
	lshpss -pvr > $raw_out		# no parm -> hpss "lshpss -pvr" command executed
else
	cp -p $inputdir/$raw_out ./$raw_out	# parm -> copy existing raw-file	# 20200109
	echo "Existing raw-output file is copied, instead of executing HPSS command." >> $logfile
fi
cat $raw_out |awk -f $AP/extract_lshpss-pvr.awk > $proc_out	# invoke awk processing 
echo "PVR LIST,"$proc_out","$raw_out >> $outfilelist		# append names of the output files to the CSV file
								# 取得情報リスト 追記
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt lshpss -pvr process completed." 	>> $logfile
# 
# ######################################################
# 2) get dumppv_pvr and process the output
#	loop needed for getting and processing output of each PVR
#	PVR毎にループして取得・加工する必要がある
echo "*** " >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvr processing started." >> $logfile
# 
if [ $uc -eq 0 ] ;			#  check command-line parameter 
then
	while read pvrname						# read one line after another from "done <" (several lines below)
									# read で1行づつ読み込む 読み込み元は done の後ろで指定
	do
		echo "PVR Name: $pvrname" >> $logfile
		pvrname_escaped=$(echo $pvrname | sed -e 's/ /_/g' -e 's/(//g' -e 's/)//g') 
							# generate a filename string from PVRname, replace spaces with underscore and remove parentheses
							#  ファイル名用文字列生成 スペースをアンダースコアに置換。括弧を削除。
		# echo $pvrname_escaped					# debug

		raw_out=raw_dumppv-pvr_$pvrname_escaped.txt		# determine the file name for raw-output of HPSS command
									# 出力ファイル名を決定
		c_str="dumppv_pvr -l '"$pvrname"' > "$raw_out		# generate a "command and options" string
									# 実行するコマンドとオプションを文字列に格納
		echo "Command to be executed: " $c_str  >> $logfile	# log "command and options" string
        	eval $c_str						# execute hpss command "dumppv_pvr -l" 
	done < $proc_out						# <=== loop input is output of the previous output
									# 1) の出力がPVR 1個/行 で、これをループの入力にしている
else
	cp -p $inputdir/raw_dumppv-pvr* .				# parm -> copy existing raw-files(for each PVR)   20200109
	echo "Existing raw-output files are copied, instead of executing HPSS command." >> $logfile
	ls -l ./raw_dumppv-pvr*.txt >> $logfile				# Files are listed into the log
fi
	
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvr outputs ended." >> $logfile
flist=$(ls raw_dumppv-pvr*.txt) 					# list up raw_dumppv-pvr_*.txt files to be processed
									# n個のraw_dumppv-pvr_*.txt を処理するためファイル名リスト取得
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvr processing started" >> $logfile
procfilelist=""

for fl in $flist					# loop for each of raw_dumppv-pvr_*.txt files
do							# raw_dumppv-pvr_*.txt ファイルの分ループする
	proc_out=$(echo $fl | sed -e 's/raw/proc/')		# determine the file name for processed-output
	procfilelist=$procfilelist","$proc_out
	
	# 20200111 Extract "Physical Volumes Pending Move" listing and remove them from "dumppv_pvr" output
	#          "Physical Volumes Pending Move"の抜き出し、及び 元ファイルからの削除 開始
	#
	#	When "Physical Volumes Pending Move" condition happens, the output of "dumppv_pvr" has the following parts, ie.
	# 	1) Normal volume list, which conatins tape volumes in the PVR specified.
	#	2) "Physical Volumes Pending Move" volume list. This list appears in every "dumppv_pvr" output.
	#	   But the list is the same in all outputs. So the list should be processed only once.
	#       In the process below, 1) is stored in the file with original filenames.
	#	2) is removed from the original output and stored in the file named as "raw_dumppv-pvr_pendingMove.txt".
	#
	#	 該当する行が発生する時は、複数のdumppv_pvr出力の全てで発生する。
	#		該当行より前の部分はそれぞれ抜き出して処理しなくてはいけない。
	#			=> 元のファイル名で保管。
	#		該当行より後の部分は殆どの場合同じ内容で、1個だけ処理しなくてはいけない。
	#			=> raw_dumppv-pvr_pendingMove.txt に保管。
	#
	r_str=$(grep -n "Physical Volumes Pending Move" $fl)	# check if the "Physical Volumes Pending Move" exists or not 
								# "Physical Volumes Pending Move"の行があるかgrepする
	rc=$?							# save return code of grep
        if [ $rc -eq 0 ]; 					# if the RC is 0 ( string "found") then
        							# grepのリターンコードが0(見つかった)ならば
        then 
        	echo "    In dumppv_pvr output, Physical Volumes Pending Move listing detected."  >> $logfile
        	echo "    Filename: "$fl                                                    >> $logfile
        	echo "    Line Number and content: " $r_str                                 >> $logfile
        	hitrownum=$(echo $r_str| awk -F':' '{print $1}')	# get the line number of "Physical Volumes Pending Move" 
        								# 見つかった行番号を取得 (行番号:見つかった行の内容 なので、":"の前を取り出す)
        	totalnumrows=$(cat $fl|wc -l)				# get the total number of lines
        								# 総行数を取得
        	cp -p $fl _dumppvWork.txt				# copy original file to work 
        								# 元ファイルをワークファイルに複写
						
        	partOne=$(($hitrownum-1))				# substract 1 from the line number which the string was found
        								# 見つかった行番号マイナス1を計算(headに使う)
        	cat _dumppvWork.txt | head -$partOne > $fl		# extract lines just before the found line and save them into the original file
        								# 見つかった行の1行前までだけを取り出して、元のファイル名で保管

        	partTwo=$(($totalnumrows-$hitrownum+1))			# calculate the line number which the string was found from the number of total lines plus 1
        								# 総行数マイナス見つかった行番号プラス1を計算(tailに使う)
        	cat _dumppvWork.txt | tail -$partTwo > raw_dumppv-pvr_pendingMove.txt	# extract lines just after the found line and save them into the other file
        										# 見つかった行以降を取り出して、専用のファイル名で保管
        fi
        # 20200111 Comptleted of Extraction of "Physical Volumes Pending Move" listing and removal from "dumppv_pvr" output
        #	   "Physical Volumes Pending Move" の抜き出しと元ファイルからの削除 終了
        #
	cat $fl | awk -f $AP/proc_dumppv_pvr.awk > $proc_out	# convert dumppv_pvr raw-output to CSV
								# dumppv_pvr 生出力をCSVに変換
done
	# 20200111 start Processing of "Physical Volumes Pending Move" output
	#          "Physical Volumes Pending Move" 関連行の加工処理 開始
	
	if [ -s raw_dumppv-pvr_pendingMove.txt ]; then		# if "Physical Volumes Pending Move" output file is found
								# "Physical Volumes Pending Move"の出力ファイルがあれば
        	echo "Physical Volumes Pending Move・・・lines are found. They are now processed here."  >> $logfile
		cat raw_dumppv-pvr_pendingMove.txt | awk -f $AP/proc_dumppv_pvr_pendingMove.awk > proc_dumppv-pvr_pendingMove.txt	# convert dumppv_pvr raw-output to CSV
																	# dumppv_pvr 生出力をCSVに変換
	fi	
	# 20200111 end Processing of "Physical Volumes Pending Move" output
	#          "Physical Volumes Pending Move" 関連行の加工処理 終了

curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvr processing completed." >> $logfile
procfilelist=$(echo $procfilelist | sed -e 's/^,//')
echo "  File name is $procfilelist"   >> $logfile
							# merge and sort proc_dumppv-pvr_*.txt files
							# 複数の proc_dumppv-pvr_*.txt ファイルをマージしてソートする
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt Merge and sort dumppv_pvr outputs" >> $logfile
proc_out=sort_dumppv-pvr.txt 					# determine the file name for processed, sort-merged output
								# 出力ファイル名を決定 ソート(proc_ ...)
cat proc_dumppv-pvr_*.txt |sort > $proc_out			# Merge and sort contents to one file
#
echo "dumppv_pvr LIST,"$proc_out","$procfilelist >> $outfilelist	# append names of the output files to the CSV file
									# 取得情報リスト 追記
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvr merged and sorted." >> $logfile
#
# ######################################################
# 3) get dumppv_pvl and filter by "Tape", "Disc" and other  2019/08/27
echo "*** "                                                           >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dumppv_pvl process started" >> $logfile
raw_out=raw_dumppv-pvl.txt		# determine the file name for raw-output of HPSS command
					# 出力ファイル名を決定 コマンド出力未加工(生)
proc_out=proc_dumppv-pvl.txt		# determine the file name for processed-output of HPSS command
					# 出力ファイル名を決定 加工後
if [ $uc -eq 0 ] ;			# check command-line parameter 
					#  コマンド実行か?
then
	dumppv_pvl > $raw_out			# # no parm -> hpss "dumppv_pvl" command executed
else
	cp -p $inputdir/$raw_out ./$raw_out	# parm -> copy existing raw-file 20200109
	echo "Existing raw-output file is copied, instead of executing HPSS command." >> $logfile
fi
cat $raw_out |awk -f $AP/proc_dumppv_pvl.awk | sort > $proc_out		# processing outputs, sort and save them 
									# 出力を加工してソート
# separate output lines by device types
# 装置タイプ毎のリストを作成する。
	# Tape 
	proc2_out=proc_dumppv-pvl_Tape.txt	# determine the file name for processed-output for Tape
						# 出力ファイル名を決定 加工後 テープのみ
	cat $proc_out  |grep  "Tape\"$" > $proc2_out 	# find lines by trailing Tape" at the end of lnie
							# 行末尾がTape" で検出
	num_recs=$(cat $proc2_out|wc -l)		# get number of lines found
							# 出力行数を取得
	echo "dumppv_pvl TapeLIST,"$proc2_out"," $num_recs","$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
													# 取得情報リスト 追記
	# Disc
	proc2_out=proc_dumppv-pvl_Disk.txt	# determine the file name for processed-output for Disk
						# 出力ファイル名を決定 加工後 ディスクのみ
	cat $proc_out  |grep  "Disk\"$" > $proc2_out 	# find lines by trailing Disk" at the end of lnie
							# 行末尾がDisk" で検出
	num_recs=$(cat $proc2_out|wc -l)		# get number of lines found
							# 出力行数を取得
	echo "dumppv_pvl DiskLIST,"$proc2_out"," $num_recs","$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
													# 取得情報リスト 追記
	# other  This may be null .... check if the lines exist or not 
	# その他 ここはもしかすると0行かもしれないので、wc -l 対策をしておく
	proc2_out=proc_dumppv-pvl_other.txt	# determine the file name for processed-output for other than Tape nor Disk
						# 出力ファイル名を決定 加工後 テープでもディスクでもない
	cat $proc_out  |grep  -v "Tape\"$"|grep -v "Disk\"$" > $proc2_out	# find lines w/o Tape" nor Disk" at the end of line
										# 末尾がDisk" でもTape"でもない行だけ検出
										
		# rm $proc2_out				# debug for the if statement below ... 1: the file does not exist (ファイルがない)
		# echo "" > $proc2_out			# debug for the if statement below ... 2: if the file size is 1. (ファイルサイズが1)
		# rm  $proc2_out; touch $proc2_out	# debug for the if statement below ... 3: if the file size is 0. (ファイルサイズが0)

	if [ -s $proc2_out ]; then		# if the file exists and its size is more than 1
						# もし、ファイルがありかつサイズが1以上なら
			# echo "Non ZERO FILE"		# debug
		num_recs=$(cat $proc2_out|wc -l)	# count number of lines
							# 行数をカウントする
			# echo $num_recs		# debug
	else
			# echo "ZERO FILE"		# dubug 
		num_recs=0		# set number of lines as 0 if the file does not exist or its size is zero.
					# ファイルがないか、あっても0バイトなら、行数は0とする
			# echo $num_recs		# dubug 
	fi	
	echo "dumppv_pvl OtherLIST,"$proc2_out"," $num_recs","$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
													# 取得情報リスト 追記
#
# ######################################################
# 4) get Storage class list, split it into Disk and Tape etc. 2019/08/28
echo "*** "                                                          >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt lshpss -sc  process started"      >> $logfile
raw_out=raw_lshpss-sc.txt		# determine the file name for raw-output of HPSS command
					# 出力ファイル名を決定 コマンド出力未加工(生)
proc_out_pre=proc_lshpss-sc		# determine the file name PREFIX for processed-output of HPSS command
					# 出力ファイル名を決定 加工後(途中まで、後半はあとから)

if [ $uc -eq 0 ] ;			# check command-line parameter 
					#  コマンド実行か?
then
	lshpss -sc  > $raw_out		# no parm -> hpss "lshpss -sc"  command executed
else
	cp -p $inputdir/$raw_out ./$raw_out	# parm -> copy existing raw-file	20200109
	echo "Existing raw-output file is copied, instead of executing HPSS command." >> $logfile
fi

# Separate output to Disk and Tape lists
# Disk と Tapeのリストが両方出てくるので、分離する
total=$(cat ./$raw_out | wc -l)				# get total number of lines
							# 全体の行数を取り出して
nu=$(cat -n ./$raw_out | grep "TAPE STORAGE CLASS LIST"| head -1|awk '{print $1}') 
					# find first line number for TAPE ( Disk class list appears first, then Tape class list follows)
				 	# 最初にDisk、その後Tapeなので、Tapeが出現する最初の行を検出
top_to_tape=$(($nu-1))			# get number of lines for Disk lists
					# Tapeの最初の行の一つ前までDiskなのでその行数を取得
tape_to_end=$(($total-$nu+1))		# number of lines for Tape lists = Total - first line number for TAPE + 1
					# Tape部分の行数 = 全体行数 - Tapeの最初の行番号+1 を計算
proc_out=$proc_out_pre"_Disk.txt"	# determine the file name for processed-output of HPSS command, DISK classes
					# Disk用の出力ファイル名を決定
	# echo $proc_out			# debug
cat ./$raw_out |head -$top_to_tape > $proc_out	# output only Disk SC List lines
						# Diskのscリストを出力

echo "lshpss -sc DiskLIST,"$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
								# 取得情報リスト 追記

proc_out=$proc_out_pre"_Tape.txt"	# determine the file name for processed-output of HPSS command, TAPE classes
					# Tape用の出力ファイル名を決定
	# echo $proc_out			# debug
cat ./$raw_out |tail -$tape_to_end > $proc_out	# output only Tape SC List lines
						# Tapeのscリストを出力

echo "lshpss -sc TapeLIST,"$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
								# 取得情報リスト 追記

# extract only 3 leftmost columns from "lshpss -sc" detail output, for Disk and Tape respectively
# Disk, Tape毎に最初の3個のカラムだけを取り出す
flist=$(ls $proc_out_pre*)
for fl in $flist
do
	tempout=_$fl.tmp					# determine temporary output file name
								# 臨時用出力ファイル名を決定
	cat $fl| awk -f $AP/extract_lshpss-sc.awk > $tempout	# extract 3 leftmost columns and save them to the temporary file
								# 最初の3カラム取り出し => 臨時ファイルに書き出し
	cp -p $fl      temp_buf.txt				# save the contents of the original file and 
								# 3カラム取り出し前ファイルは一旦複写して内容を待避
	cp -p $tempout $fl					# replace the file with the extarcted contents ( 3 leftmost columns)
								# 3カラム取り出し済みファイルを本来のproc_....ファイル名とする
	cp -p temp_buf.txt $tempout				# save the contents of the original file with a name of .... .tmp for debug
								# 3カラム取り出し前ファイル はデバッグ用に ....tmp として保存
done
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt lshpss -sc Disk, Tape separated and processed."      >> $logfile
#
# ######################################################
# 5) get dump_sspvs -i 1 
echo "*** "                                                          >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dump_sspvs -i 1  processing started"      >> $logfile
raw_out=raw_dump_sspvs-i.txt		# determine the file name for raw-output of HPSS command
					# 出力ファイル名を決定 コマンド出力未加工(生)
proc_out=proc_dump_sspvs-i_detail.txt	# determine the file name for processed-output of HPSS command
					# 出力ファイル名を決定 加工後
if [ $uc -eq 0 ] ;			# check command-line parameter 
					#  コマンド実行か?
then
	dump_sspvs -i 1 > $raw_out		# no parm -> hpss "dump_sspvs -i 1" executed
else
	cp -p $inputdir/$raw_out ./$raw_out	# parm -> copy existing raw-file	# 20200109
	echo "Existing raw-output file is copied, instead of executing HPSS command." >> $logfile
fi

# ###################
# 5-1) First part of dump_sspvs -i 1, detail by volume 
#      dump_sspvs -i 1 の前段、ボリューム毎詳細 

total=$(cat ./$raw_out | wc -l)		# get total number of lines
					# 全体の行数を取り出して
nu=$(cat -n ./$raw_out | grep "                             -------------------------- --------------------------"| head -1|awk '{print $1}')  
					# get the line number of the separator line of first part
					# 最初にテープボリューム明細があるので、明細終了/区切り行を検出
top_to_detailend=$(($nu-1))		# calculate number of lines just before separtor line
					# 見つかった行の一つ前まで明細
detaillines=$(($top_to_detailend-4))	# suvstract number of lines by 4 because the fist 4 lines are headers.
					# ファイル最初の4行はヘッダーなので、差し引く

cat ./$raw_out | head -$top_to_detailend| tail -$detaillines > _sspvs-i_detail.txt	# extract detail lines to the Temporary file
											# 詳細行だけを抜き出して一時ファイルに書く
cat _sspvs-i_detail.txt | awk -f $AP/extract_dump_sspvs-i_detail.awk > $proc_out	# process with an awk script and save
											# awkで加工して出力

echo "dump_sspvsLIST,"$proc_out","$raw_out >> $outfilelist	# append names of the output files to the CSV file
								# 取得情報リスト 追記
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dump_sspvs -i 1  detail by volume processed"      >> $logfile

# ###################
# 5-2) Second part of dump_sspvs -i 1, summary by SC 	Not implemented 
#      dump_sspvs -i 1 の後段、SC毎合計			実装されていない
# ######################################################
# 6) get dump_sspvs -i 1 --lstapes-format
echo "*** "                                                          >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dump_sspvs -i 1 --lstapes-format  started"      >> $logfile
raw_out=raw_dump_sspvs-i-lstapes-format.txt	# determine the file name for raw-output of HPSS command
						# 出力ファイル名を決定 コマンド出力未加工(生)
proc_out=proc_dump_sspvs-i-lstapes-format.txt	# determine the file name for processed-output of HPSS command
						# 出力ファイル名を決定 加工後
if [ $uc -eq 0 ] ;			# check command-line parameter 
then
	dump_sspvs -i 1 --lstapes-format > $raw_out			# no parm -> hpss "dump_sspvs -i 1 --lstapes-format" executed
else
	cp -p $inputdir/$raw_out ./$raw_out	# parm -> copy existing raw-file	# 20200109
	echo "Existing raw-output file is copied, instead of executing HPSS command." >> $logfile
fi

cat ./$raw_out | awk -f $AP/extract_dump_sspvs-i-lstapes-format.awk > $proc_out		# process with an awk script and save
echo "dump_sspvs--lstapesLIST,"$proc_out","$raw_out >> $outfilelist		# append names of the output files to the CSV file
										# 取得情報リスト 追記
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt dump_sspvs -i 1 --lstapes-format processing completed"      >> $logfile

# ######################################################
# 7) make Tape volume list, and compare it with the previous exceution list
#
echo "*** "                                                          >> $logfile
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt comparing the list of labels with the previous output"      >> $logfile
# 7-1) strip volume ID from dumppv-pvr output
#      dumppv-pvr出力から、今回のvolumeリストファイルを作成
cat sort_dumppv-pvr.txt | awk '{vol=substr($0,2,8);print vol}' > volumelist.txt
#
# 7-2) check and find previoius execution list files
#      前回の実行結果ファイルがあるか探す
prev_dir=""		# directory to hold output files of the last run 	前回のディレクトリー
prev_listfile=""	# filename of volumelist.txt of the last run		前回のvolumeリストファイル (ファイル名は固定 volumelist.txt)
prev_accumfile=""	# filename of volaccumlist.txt(accumulated results) of the last run	前回の累積ファイル (ファイル名は固定 volaccumlist.txt)
prev_dir_count=0
prev_listfile_count=0
prev_accumfile_count=0

c=$(ls -F .. | grep "$datadir_pre" | grep "/" | awk 'BEGIN{FS="/"};{print  $1}'| tail -2 | wc -l)   
	# test if there are directories to hold previous results.  
	# If number is more than or equal to 2, Yes. 
	# If number is 1, No. The only directory which is found is the current one. 
	# 0 should not occur, because the directory was made at the begining of this shell script.
	# 前の実行結果ディレクトリーがあるか? 2なら前のdirがある。1は今回の自分のdirしかない。0はあり得ない(スクリプト内で作っているので)。
if [ $c -eq 2 ]; then
	prev_dir=$(ls -F .. | grep "$datadir_pre" | grep "/" | awk 'BEGIN{FS="/"};{print  $1}'|tail -2|head -1)	# identify the last executed directory
														# 前回のディレクトリーを特定
		# identify directories using "ls -F", filter with "/", get the last one and remove "/"
		# ディレクトリー以外のファイルが検出されないように "ls -F"して "/"がつくものだけを探し、awkで末尾の"/"を外す
	prev_dir="../"$prev_dir						# place ".." to specify the directory which is neighbor upon the current one.
									# "../"を前置して、隣にあることを示す。
	curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Directory of the last run is found.           " $prev_dir   >> $logfile
	prev_dir_count=1
	if [ -s $prev_dir"/"volumelist.txt ]; then			# if volumelist.txt of the last run is found and its size is more than 1
									# 前回のvolumeリストファイルがありかつサイズが1以上なら
			prev_listfile=$prev_dir"/volumelist.txt"
			curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   volumelist.txt of the last run is found.    " $prev_listfile   >> $logfile
			prev_listfile_count=1
		else							# if not found
			curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   volumelist.txt of the last run is NOT found." $prev_listfile   >> $logfile
			prev_listfile_count=0
	fi	
	if [ -s $prev_dir"/"volaccumlist.txt ]; then			# if volaccumlist.txt of the last run is found and its size is more than 1
									# 前回の累積ファイルがありかつサイズが1以上なら
			prev_accumfile=$prev_dir"/volaccumlist.txt"
			curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   volaccumlist.txt of the last run is found.  " $prev_accumfile  >> $logfile
			prev_accumfile_count=1
		else							# if not found
			curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   volaccumlist.txt of the last run is NOT found." $prev_accumfile  >> $logfile
			prev_accumfile_count=0
	fi	
	
  	else
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Directory of the last run is NOT found. This run is the first one." $prev_dir   >> $logfile
		prev_dir_count=0
fi
# 7-3) get diffirence between the current and the last run
if [ $prev_listfile_count -eq 1 ] ;	# if the last run output exists
					# もし前回のリストがあれば
	then
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   compare with the last run result ..."  >> $logfile
		diff $prev_listfile ./volumelist.txt > difflist.txt	# get diff (if there is no difference, the file is made with size of 0)
									# diff を採取 (差が無くても、ファイルはできる(サイズ=0))
		cat difflist.txt | awk -f $AP/diff_convert.awk -v d=$dt_start_separator -v t=$hm_start_separator > ./diffconverted.txt
									# invoke awk to convert the lines with addition/removal and date/time 
									# 追加削除・日時付きファイルに加工
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Volume addition/removal detection completed.    ./diffconverted.txt"  >> $logfile

		if [ $prev_accumfile_count -eq 1 ] ;
			then
				cp -p $prev_accumfile ./volaccumlist.txt		# copy volumelist.txt of the last run to the current run directory
											# 前回の累積ファイルを現在の作業ディレクトリーに複写
		fi
		# arrange the lines in reverse to make the newer lines first
		# より新しい行が上になるように逆順にする
		tac ./diffconverted.txt > ./diffconverted_rev.txt			# tac output the lines in reverse fashion
											# tac で 今回の増減加工済みファイルを降順にする
		cat ./diffconverted_rev.txt ./volaccumlist.txt > ./_volaccumlist.txt 	# concatenate the current result and the accumulated results into the work file
											# 今回の増減加工済みファイルの下に前回の累積を連結してワークファイルに書く
		cat ./_volaccumlist.txt > ./volaccumlist.txt				# override the accumulated history with the updated one in the work file
											# 前回の累積を、ワークファイルで上書きする
		
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Volume addition/removal history processing completed.       ./volaccumlist.txt"  >> $logfile
		
	else				# else means that this is the first run
					# もし前回のリストがなければ、今回が最初なので
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   This is the frst run."  >> $logfile
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   All volumes are treated as newly added ones."  >> $logfile
		
		cat ./volumelist.txt | awk '{print "> "$1}' > difflist.txt 	# add ">" to all volumes to make the list similar to diff output
										# awk で全ボリュームに">"を付加して、diffの出力に見せかける
		cat difflist.txt | awk -f $AP/diff_convert.awk -v d=$dt_start_separator -v t=$hm_start_separator > ./diffconverted.txt
									# invoke awk to convert the lines with addition/removal and date/time 
									# 追加削除・日時付きファイルに加工
		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Volume addition/removal detection completed.    ./diffconverted.txt"  >> $logfile

		# arrange the lines in reverse to make the newer lines first
		# より新しい行が上になるように逆順にする
		tac ./diffconverted.txt > ./diffconverted_rev.txt			# tac output the lines in reverse fashion
											# tac で 今回の増減加工済みファイルを降順にする
		cat ./diffconverted_rev.txt  > ./_volaccumlist.txt 	                # copy the current result into the work file
											# 今回の増減加工済みファイルをワークファイルに書く
		cat ./_volaccumlist.txt > ./volaccumlist.txt				# 前回の累積はもう要らないので上書きする

		curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt   Volume addition/removal history processing completed.       ./volaccumlist.txt"  >> $logfile
fi
echo "volume add_remove,volaccumlist.txt,diffconverted.txt" >> $outfilelist	# append names of the output files to the CSV file
										# 取得情報リスト 追記
#
# ######################################################
# 8) copy the template of HPSS Tape Catalog Excel file with a name of executed date and time
#    HPSSテープカタログのテンプレートExcelファイルを、実行日時付きの名前で複写
cp -p $AP/HPSS_TapeCatalog_Template.xlsm ./HPSS_TapeCatalog_$dt_start$hm_start.xlsm
curdt=$(date +"%Y/%m/%d %H:%M:%S"); echo "$curdt HPSS Tape Catalog Excel file is copied from the template. "  >> $logfile
#

# ######################################################
# 9) end
echo "$cmdname ended."                                      >> $logfile # 終了

